@@ -1,10 +1,18 @@
-import functools
+import typing as tp
+import collections
+import copy
+import logging
 import threading
 
+from ..decorators import wraps
+
 __all__ = [
-    'Monitor', 'RMonitor'
+    'Monitor', 'RMonitor', 'MonitorDict', 'MonitorList'
 ]
 
+K, V, T = tp.TypeVar('K'), tp.TypeVar('V'), tp.TypeVar('T')
+logger = logging.getLogger(__name__)
+
 
 class Monitor:
     """
@@ -43,7 +51,7 @@ class Monitor:
         may vary
         """
 
-        @functools.wraps(fun)
+        @wraps(fun)
         def monitored(*args, **kwargs):
             # noinspection PyProtectedMember
             with args[0]._monitor_lock:
@@ -104,6 +112,29 @@ class Monitor:
             self.foo._monitor_lock.release()
             return False
 
+    @classmethod
+    def synchronize_on(cls, monitor):
+        """
+        A decorator for locking on non-self Monitor objects
+
+        Use it like:
+
+        >>> class MasterClass(Monitor):
+        >>>     def get_object(self):
+        >>>         class SlaveClass:
+        >>>             @Monitor.synchronize_on(self)
+        >>>             def get_object(self2):
+        >>>                 ...
+        >>>         return SlaveClass
+        """
+        def outer(fun):
+            @wraps(fun)
+            def inner(*args, **kwargs):
+                with cls.acquire(monitor):
+                    return fun(*args, **kwargs)
+            return inner
+        return outer
+
 
 class RMonitor(Monitor):
     """
@@ -111,6 +142,48 @@ class RMonitor(Monitor):
     """
 
     def __init__(self):
-        # todo refactor so that a Lock is not created uselessly
-        super(RMonitor, self).__init__()
         self._monitor_lock = threading.RLock()
+
+
+class MonitorList(tp.Generic[T], collections.UserList, Monitor):
+    """A list that is also a monitor"""
+    def __init__(self, *args):
+        collections.UserList.__init__(self, *args)
+        Monitor.__init__(self)
+
+    def __copy__(self):
+        return MonitorList(copy.copy(self.data))
+
+    def __deepcopy__(self, memodict={}):
+        return MonitorList(copy.deepcopy(self.data, memo=memodict))
+
+    def __getitem__(self, item: tp.Union[slice, int]) -> T:
+        return self.data[item]
+
+    def __setitem__(self, key: int, value: T) -> None:
+        self.data[key] = value
+
+    def __delitem__(self, key: tp.Union[slice, int]) -> None:
+        del self.data[key]
+
+
+class MonitorDict(tp.Generic[K, V], collections.UserDict, Monitor):
+    """A dict that is also a monitor"""
+    def __init__(self, *args, **kwargs):
+        collections.UserDict.__init__(self, *args, **kwargs)
+        Monitor.__init__(self)
+
+    def __getitem__(self, item: K) -> V:
+        return self.data[item]
+
+    def __setitem__(self, key: K, value: V) -> None:
+        self.data[key] = value
+
+    def __delitem__(self, key: K) -> None:
+        del self.data[key]
+
+    def __copy__(self):
+        return MonitorDict(copy.copy(self.data))
+
+    def __deepcopy__(self, memodict={}):
+        return MonitorDict(copy.deepcopy(self.data, memo=memodict))
