@@ -6,9 +6,14 @@ import logging
 import operator
 import time
 import typing as tp
+from abc import ABCMeta, abstractmethod
+
+from ..decorators import wraps
 
 logger = logging.getLogger(__name__)
 
+T = tp.TypeVar('T')
+
 __all__ = [
     'Heap',
     'SetHeap',
@@ -17,7 +22,7 @@ __all__ = [
 ]
 
 
-class OmniHashableMixin:
+class OmniHashableMixin(metaclass=ABCMeta):
     """
     A mix-in. Provides hashing and equal comparison for your own class using specified fields.
 
@@ -39,7 +44,11 @@ class OmniHashableMixin:
 
     for this to work in your class
     """
-    _HASH_FIELDS_TO_USE = []
+    @property
+    @abstractmethod
+    def _HASH_FIELDS_TO_USE(self) -> tp.List[str]:
+        """Return the names of properties that will be used for __eq__ and __hash__"""
+        return []
 
     def __hash__(self):
         return functools.reduce(operator.xor, (hash(getattr(self, field_name))
@@ -65,17 +74,14 @@ class OmniHashableMixin:
 
 
 def _extras_to_one(fun):
-    @functools.wraps(fun)
+    @wraps(fun)
     def inner(self, a, *args, **kwargs):
         return fun(self, ((a,) + args) if len(args) > 0 else a, **kwargs)
 
     return inner
 
 
-HeapVar = tp.TypeVar('T')
-
-
-class Heap(collections.UserList, tp.Generic[HeapVar]):
+class Heap(collections.UserList, tp.Generic[T]):
     """
     Sane heap as object - not like heapq.
 
@@ -85,16 +91,16 @@ class Heap(collections.UserList, tp.Generic[HeapVar]):
     Not thread-safe
     """
 
-    def __init__(self, from_list: tp.Optional[tp.Iterable[HeapVar]] = None):
+    def __init__(self, from_list: tp.Optional[tp.Iterable[T]] = None):
         super().__init__(from_list)
         heapq.heapify(self.data)
 
-    def push_many(self, items: tp.Iterable[HeapVar]) -> None:
+    def push_many(self, items: tp.Iterable[T]) -> None:
         for item in items:
             self.push(item)
 
     @_extras_to_one
-    def push(self, item: HeapVar):
+    def push(self, item: T) -> None:
         """
         Use it like:
 
@@ -106,21 +112,16 @@ class Heap(collections.UserList, tp.Generic[HeapVar]):
         """
         heapq.heappush(self.data, item)
 
-    def __copy(self, op) -> 'Heap':
-        h = Heap()
-        h.data = op(self.data)
-        return h
-
-    def __deepcopy__(self, memo) -> 'Heap':
-        return self.__copy(copy.deepcopy)
+    def __deepcopy__(self, memodict={}) -> 'Heap':
+        return self.__class__(copy.deepcopy(self.data, memo=memodict))
 
     def __copy__(self) -> 'Heap':
-        return self.__copy(copy.copy)
+        return self.__class__(copy.copy(self.data))
 
-    def __iter__(self) -> tp.Iterator[HeapVar]:
+    def __iter__(self) -> tp.Iterator[T]:
         return self.data.__iter__()
 
-    def pop(self) -> HeapVar:
+    def pop(self) -> T:
         """
         Return smallest element of the heap.
 
@@ -128,8 +129,8 @@ class Heap(collections.UserList, tp.Generic[HeapVar]):
         """
         return heapq.heappop(self.data)
 
-    def filter_map(self, filter_fun: tp.Optional[tp.Callable[[HeapVar], bool]] = None,
-                   map_fun: tp.Optional[tp.Callable[[HeapVar], tp.Any]] = None):
+    def filter_map(self, filter_fun: tp.Optional[tp.Callable[[T], bool]] = None,
+                   map_fun: tp.Optional[tp.Callable[[T], tp.Any]] = None):
         """
         Get only items that return True when condition(item) is True. Apply a
          transform: item' = item(condition) on
@@ -147,7 +148,7 @@ class Heap(collections.UserList, tp.Generic[HeapVar]):
         """
         return len(self.data) > 0
 
-    def iter_ascending(self) -> tp.Iterable[HeapVar]:
+    def iter_ascending(self) -> tp.Iterable[T]:
         """
         Return an iterator returning all elements in this heap sorted ascending.
         State of the heap is not changed
@@ -156,7 +157,7 @@ class Heap(collections.UserList, tp.Generic[HeapVar]):
         while heap:
             yield heapq.heappop(heap)
 
-    def iter_descending(self) -> tp.Iterable[HeapVar]:
+    def iter_descending(self) -> tp.Iterable[T]:
         """
         Return an iterator returning all elements in this heap sorted descending.
         State of the heap is not changed.
@@ -177,11 +178,11 @@ class Heap(collections.UserList, tp.Generic[HeapVar]):
     def __repr__(self) -> str:
         return u'<satella.coding.Heap>'
 
-    def __contains__(self, item: HeapVar) -> bool:
+    def __contains__(self, item: T) -> bool:
         return item in self.data
 
 
-class SetHeap(Heap, tp.Generic[HeapVar]):
+class SetHeap(Heap):
     """
     A heap with additional invariant that no two elements are the same.
 
@@ -190,25 +191,25 @@ class SetHeap(Heap, tp.Generic[HeapVar]):
     #notthreadsafe
     """
 
-    def __init__(self, from_list: tp.Optional[tp.Iterable[HeapVar]] = None):
+    def __init__(self, from_list: tp.Optional[tp.Iterable[T]] = None):
         super().__init__(from_list=from_list)
         self.set = set(self.data)
 
-    def push(self, item: HeapVar):
+    def push(self, item: T):
         if item not in self.set:
             super().push(item)
             self.set.add(item)
 
-    def pop(self) -> HeapVar:
+    def pop(self) -> T:
         item = super().pop()
         self.set.remove(item)
         return item
 
-    def __contains__(self, item: HeapVar) -> bool:
+    def __contains__(self, item: T) -> bool:
         return item in self.set
 
-    def filter_map(self, filter_fun: tp.Optional[tp.Callable[[HeapVar], bool]] = None,
-                   map_fun: tp.Optional[tp.Callable[[HeapVar], tp.Any]] = None):
+    def filter_map(self, filter_fun: tp.Optional[tp.Callable[[T], bool]] = None,
+                   map_fun: tp.Optional[tp.Callable[[T], tp.Any]] = None):
         super().filter_map(filter_fun=filter_fun, map_fun=map_fun)
         self.set = set(self.data)
 
@@ -238,7 +239,7 @@ class TimeBasedHeap(Heap):
     def __repr__(self):
         return '<satella.coding.TimeBasedHeap with %s elements>' % (len(self.data),)
 
-    def items(self) -> tp.Iterable[HeapVar]:
+    def items(self) -> tp.Iterable[T]:
         """
         Return an iterable, but WITHOUT timestamps (only items), in
         unspecified order
@@ -252,8 +253,8 @@ class TimeBasedHeap(Heap):
         self.default_clock_source = default_clock_source or time.monotonic
         super().__init__(from_list=())
 
-    def put(self, timestamp_or_value: tp.Union[tp.Tuple[tp.Union[float, int], HeapVar]],
-            value: tp.Optional[HeapVar] = None) -> None:
+    def put(self, timestamp_or_value: tp.Union[tp.Tuple[tp.Union[float, int], T]],
+            value: tp.Optional[T] = None) -> None:
         """
         Put an item on heap.
 
@@ -268,7 +269,7 @@ class TimeBasedHeap(Heap):
         self.push((timestamp, item))
 
     def pop_less_than(self, less: tp.Optional[tp.Union[int, float]] = None) -> tp.Generator[
-        HeapVar, None, None]:
+        T, None, None]:
         """
         Return all elements less (sharp inequality) than particular value.
 
@@ -288,7 +289,7 @@ class TimeBasedHeap(Heap):
                 return
             yield self.pop()
 
-    def remove(self, item: HeapVar) -> None:
+    def remove(self, item: T) -> None:
         """
         Remove all things equal to item
         """
